package com.tuoheng.mapper;

import com.tuoheng.model.po.UserPo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

/**
 * @author chenjiandong
 * @description: TODO
 * @date 2022/10/8 11:59
 */
@Mapper
public interface ClientUserMapper {

    int insertClientUser(UserPo userPo);

    int judgeCreateByUserName(String username);

    UserPo getUserByUserName(String userName);

    int updatePass(UserPo userPo);

    UserPo selectByUserId(@Param("userId") Long userId);

}
